/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.session;

import java.io.Serializable;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

import kieker.webgui.beans.application.GlobalPropertiesBean;

/**
 * The class {@link CurrentConfigurationBean} is a session-scoped JSF bean, containing the properties and configurations of the current (session) user.<br>
 * It is session-scoped as only one instance per user is necessary.
 * 
 * @author Nils Christian Ehmke
 */
@ManagedBean
@SessionScoped
public final class CurrentConfigurationBean implements Serializable {

	private static final long serialVersionUID = 4226402116177851270L;
	private String lookAndFeel;
	private String gridColor;
	private int gridSize;

	@ManagedProperty(value = "#{globalPropertiesBean}")
	private GlobalPropertiesBean globalPropertiesBean;

	/**
	 * Default constructor. <b>Do not use this constructor. This bean is JSF managed.</b>
	 */
	public CurrentConfigurationBean() {
		// No code necessary
	}

	/**
	 * This method initializes the bean. In fact it loads the default values before trying to load the values of the user from their cookies. <b>Do not call this
	 * method. The method is managed by the class loader.</b>
	 */
	@PostConstruct
	protected void initialize() {
		this.loadDefaultValues();
		this.loadValuesFromCookies();
	}

	/**
	 * The getter for the property {@link CurrentConfigurationBean#globalPropertiesBean}.
	 * 
	 * @return The current value for the property.
	 */
	public GlobalPropertiesBean getGlobalPropertiesBean() {
		synchronized (this) {
			return this.globalPropertiesBean;
		}
	}

	/**
	 * The setter for the property {@link CurrentConfigurationBean#globalPropertiesBean}.
	 * 
	 * @param globalPropertiesBean
	 *            The new value for the property.
	 */
	public void setGlobalPropertiesBean(final GlobalPropertiesBean globalPropertiesBean) {
		synchronized (this) {
			this.globalPropertiesBean = globalPropertiesBean;
		}
	}

	/**
	 * The getter for the property {@link CurrentConfigurationBean#lookAndFeel}.
	 * 
	 * @return The current value for the property.
	 */
	public String getLookAndFeel() {
		synchronized (this) {
			return this.lookAndFeel;
		}
	}

	/**
	 * The setter for the property {@link CurrentConfigurationBean#lookAndFeel}.
	 * 
	 * @param lookAndFeel
	 *            The new value for the property.
	 */
	public void setLookAndFeel(final String lookAndFeel) {
		synchronized (this) {
			this.lookAndFeel = lookAndFeel;
			CurrentConfigurationBean.saveValueInCookie(this.globalPropertiesBean.getThemeCookieName(), lookAndFeel);
		}
	}

	/**
	 * The getter for the property {@link CurrentConfigurationBean#gridColor}.
	 * 
	 * @return The current value for the property.
	 */
	public String getGridColor() {
		synchronized (this) {
			return this.gridColor;
		}
	}

	/**
	 * The setter for the property {@link CurrentConfigurationBean#gridColor}.
	 * 
	 * @param gridColor
	 *            The new value for the property.
	 */
	public void setGridColor(final String gridColor) {
		synchronized (this) {
			this.gridColor = gridColor;
			CurrentConfigurationBean.saveValueInCookie(this.globalPropertiesBean.getAnalysisEditorGridColorCookieName(), gridColor);
		}
	}

	/**
	 * The getter for the property {@link CurrentConfigurationBean#gridSize}.
	 * 
	 * @return The current value for the property.
	 */
	public int getGridSize() {
		synchronized (this) {
			return this.gridSize;
		}
	}

	/**
	 * The setter for the property {@link CurrentConfigurationBean#gridSize}.
	 * 
	 * @param gridSize
	 *            The new value for the property.
	 */
	public void setGridSize(final int gridSize) {
		synchronized (this) {
			this.gridSize = gridSize;
			CurrentConfigurationBean.saveValueInCookie(this.globalPropertiesBean.getAnalysisEditorGridSizeCookieName(), Integer.toString(gridSize));
		}
	}

	/**
	 * This method tries to load the default values of the properties from the {@link GlobalPropertiesBean} and the faces context.
	 */
	private void loadDefaultValues() {
		synchronized (this) {
			// Get the parameters within the current context.
			final Map<String, String> params = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap(); // NOPMD (No concurrent access)
			// Try to find the default theme within the parameters.
			if (params.containsKey(this.globalPropertiesBean.getFacesContextThemeKey())) {
				this.lookAndFeel = params.get(this.globalPropertiesBean.getFacesContextThemeKey());
			} else {
				// Use the default theme.
				this.lookAndFeel = this.globalPropertiesBean.getDefaultTheme();
			}

			this.gridColor = this.globalPropertiesBean.getAnalysisEditorDefaultGridColor();
			this.gridSize = Integer.parseInt(this.globalPropertiesBean.getAnalysisEditorDefaultGridSize());
		}
	}

	/**
	 * This method tries to load the values of the properties from the user's cookies.
	 */
	private void loadValuesFromCookies() {
		synchronized (this) {
			// Get the available cookies
			final Map<String, Object> cookies = FacesContext.getCurrentInstance().getExternalContext().getRequestCookieMap(); // NOPMD (No concurrent access)

			// Now load the values if available
			if (cookies.containsKey(this.globalPropertiesBean.getThemeCookieName())) {
				this.lookAndFeel = ((Cookie) cookies.get(this.globalPropertiesBean.getThemeCookieName())).getValue();
			}
			if (cookies.containsKey(this.globalPropertiesBean.getAnalysisEditorGridColorCookieName())) {
				this.gridColor = ((Cookie) cookies.get(this.globalPropertiesBean.getAnalysisEditorGridColorCookieName())).getValue();
			}
			if (cookies.containsKey(this.globalPropertiesBean.getAnalysisEditorGridSizeCookieName())) {
				this.gridSize = Integer.parseInt(((Cookie) cookies.get(this.globalPropertiesBean.getAnalysisEditorGridSizeCookieName())).getValue());
			}
		}
	}

	/**
	 * This method saves the given values in a cookie with the given name, using the maximal available age for the cookie.
	 * 
	 * @param cookieName
	 *            The name of the cookie.
	 * @param cookieValue
	 *            The value for the cookie.
	 */
	private static void saveValueInCookie(final String cookieName, final String cookieValue) {
		// Create the cookie and make sure that it will be saved for a year (maximum age)
		final Cookie cookie = new Cookie(cookieName, cookieValue);
		cookie.setMaxAge(60 * 60 * 24 * 365);

		// Deliver the cookie
		final HttpServletResponse response = (HttpServletResponse) FacesContext.getCurrentInstance().getExternalContext().getResponse();
		response.addCookie(cookie);
	}
}
